package koboldsim

import (
	"git.chromaticdragon.app/kistaro/CardSimEngine/cardsim"
)

var cards = []Card{
	&SwitchingCard{ // Card 1
		Name:  cardsim.MsgStr("Warborn"),
		Desc:  cardsim.MsgStr(" A surge of anti-kobold sentiment has been reported by your spies on the surface and your military is concerned that anti-kobold vigilantes will attack the tunnels in the near-future.  They want to build up now while doing so is safe."),
		After: ShuffleIntoBottomHalf,
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your war chief paces irritably.  "We have terrible threats facing us.  It's imperative that we build up our numbers in both the military and the domestic sense.  I want creches under military control and a good hunting crew to supply them with food."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your war chief is presently monitoring the situation, building up your military, and securing your creches."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation += 100
					p.Stats.ScavengingIncome += 0.01
					p.Stats.WarExpense += 0.02
					p.Stats.FoodSupply += 0.01
					p.Stats.ForeignRelations -= 0.02
					return cardsim.MsgStr("Kobolds are known to be born warriors."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation -= 100
					p.Stats.ScavengingIncome -= 0.01
					p.Stats.WarExpense -= 0.02
					p.Stats.FoodSupply -= 0.01
					p.Stats.ForeignRelations += 0.02
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your head miner considers the matter worriedly.  "Creches under military control?  No.  That would invite chaos.  We need to dig deeper; we can have a peaceful, orderly society if we just get far enough away from surfacers."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your head miner is presently leading a project to dig as far away from the surface as possible."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation += 40
					p.Stats.MiningIncome += 0.02
					p.Stats.BureaucracyExpense += 0.01
					p.Stats.FoodSupply -= 0.01
					p.Stats.Secrecy += 0.02
					p.Stats.PointOfDimReturns += 0.01
					return cardsim.MsgStr("Kobolds are known to be cowards hiding in the dark."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation -= 40
					p.Stats.MiningIncome -= 0.02
					p.Stats.BureaucracyExpense -= 0.01
					p.Stats.FoodSupply += 0.01
					p.Stats.Secrecy -= 0.02
					p.Stats.PointOfDimReturns += 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your nursery director is incensed.  "Creches under military control?  Never!  Let young kobolds play!  In fact, cut the military just for suggesting this.  The threats facing us are completely overstated."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Military funding has been diverted into early childhood education."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 40
					p.Stats.ScavengingIncome -= 0.01
					p.Stats.WarExpense -= 0.02
					p.Stats.EducationExpense += 0.01
					p.Stats.FoodSupply -= 0.01
					p.Stats.ForeignRelations += 0.02
					p.Stats.PointOfDimReturns += 0.01
					return cardsim.MsgStr("An undefended hunting outpost near the surface was recently wiped out by a raid."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 40
					p.Stats.ScavengingIncome += 0.01
					p.Stats.WarExpense += 0.02
					p.Stats.EducationExpense -= 0.01
					p.Stats.FoodSupply += 0.01
					p.Stats.ForeignRelations -= 0.02
					p.Stats.PointOfDimReturns -= 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("This isn't about a disaster and can probably be safely dismissed."),
					Do: func(p *Player) (cardsim.Message, error) {
						p.Stats.BasePopulation += 20
						return cardsim.MsgStr("Creche control doesn't shift that easily."), nil
					},
					Undo: func(p *Player) error {
						p.Stats.BasePopulation -= 20
						return nil
					},
					CanDo: YesWeCan,
				},
				Variants: []Policy{
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Rejecting this issue will also reject the military's natalist stance."),
						Do:            OverrideMsg(cardsim.MsgStr("Militant natalism has been reduced by policy.")),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr(`"Dig deeper" pressures in your nation may be excessive.`),
						Do:            OverrideMsg(cardsim.MsgStr("Some of the lower depths are being abandoned.")),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Near-surface patrols may need to be increased."),
						Do:            OverrideMsg(cardsim.MsgStr("More and better-armed hunting outposts are being established.")),
					},
					&BasicPolicy{
						EnactedDesc: cardsim.MsgStr("This isn't about a disaster and can probably continue to be safely dismissed."),
						Do:          OverrideMsg(cardsim.MsgStr("Creche control doesn't shift that easily.")),
					},
				},
			},
		}, // End of "Warborn" policies
	}, // End of "Warborn" card
	&SwitchingCard{ // Card 2
		Name:  cardsim.MsgStr("International Festival of Bureaucracy"),
		Desc:  cardsim.MsgStr(" Good times are upon us! A great festival has been declared between many kobold nations, celebrating the orderly nature of the kobold soul!  That's right, it's the International Festival of Bureaucracy!"),
		After: ShuffleIntoBottomHalf,
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your Minister of Administration is practically jumping for joy. "This is our opportunity to prove that we're a prosperous society and attract some fresh blood!  We need to raise salaries in the bureaucracy, grant some time off to our bureaucrats to make sure they can attend, and try to hire talent away from other nations.  We'll forge productive trade relations by this, you'll see!`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your bureaucrats are really looking forward to attending, where they can boast about how the festival boosted their salaries."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation += 80
					p.Stats.ScavengingIncome += 0.01
					p.Stats.BureaucracyExpense += 0.03
					p.Stats.FoodSupply += 0.01
					p.Stats.PointOfDimReturns += 0.01
					return cardsim.MsgStr("Bureaucrats are considered pillars of society."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation -= 80
					p.Stats.ScavengingIncome -= 0.01
					p.Stats.BureaucracyExpense -= 0.03
					p.Stats.FoodSupply -= 0.01
					p.Stats.PointOfDimReturns -= 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your Minister of Finance pulls you aside. "This is a good opportunity to head-hunt.  Bureaucrats from many nations will be attending.  There's got to be some who'll work for less than the ones we've got.  There are some terribly threatened or impoverished communities among kobolds.`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your nation's policy is clear: the festival of bureaucracy is a chance to trade workers with other kobold nations in your attempts to build a more efficient government."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation += 20
					p.Stats.BureaucracyExpense -= 0.01
					return cardsim.MsgStr("Immigrant bureaucrats complain about being under-appreciated."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation -= 20
					p.Stats.BureaucracyExpense += 0.01
					return nil
				},
				CanDo: func(b *BasicPolicy, p *Player) bool {
					return p.Stats.BureaucracyExpense >= 0.02 && b.LastEnactedIdx != 3
				},
				//				CanDo: func(p *Player) bool {
				//					if p.Stats.GovBureaucracyExpense.Value >= 0.02 {
				//						return true
				//					}
				//					return false
				//				},
				// Original implementation preserved in the hopes that I'll learn this stuff.  This is equivalent to the shorter code above.  I'll probably remove this comment eventually.  Sincerely, Rakeela.
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your Minister of Finance greets you exuberantly. "We've trimmed the wages of the bureaucracy beautifully, but maybe we can scout some outright volunteers.  Some nations are absolute disasters, after all.`),
				EnactedDesc:   cardsim.MsgStr(`[current policy] It's hard to find volunteer workers at the festival, but your nation is doing its best.`),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation += 10
					p.Stats.BureaucracyExpense -= 0.02
					p.Stats.PointOfDimReturns -= 0.01
					return cardsim.MsgStr("The local bureaucracy is staffed by volunteer labor."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation -= 10
					p.Stats.BureaucracyExpense += 0.02
					p.Stats.PointOfDimReturns += 0.01
					return nil
				},
				CanDo: func(b *BasicPolicy, p *Player) bool {
					return p.Stats.BureaucracyExpense < 0.02 && b.LastEnactedIdx != 3
				},
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`One of your non-bureaucrat friends has been in a foul temper.  "This festival is everything that's wrong with our society. Life isn't about filling out forms. We ought to snub this festival and outright fire some bureaucrats."`),
				EnactedDesc:   cardsim.MsgStr("Your nation currently bans the International Festival of Bureaucracy.  There will be no local celebration without a policy change."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 80
					p.Stats.BureaucracyExpense -= 0.04
					return cardsim.MsgStr("A wave of bureaucrats just emigrated along with kobolds incensed by the nation's lack of respect for administration."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 80
					p.Stats.BureaucracyExpense += 0.04
					return nil
				},
				CanDo: YesWeCan,
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("Some kobolds will attend the festival on their own, and we may face desertions for our lack of interest."),
					EnactedDesc:   cardsim.MsgStr("Some kobolds will attend the festival on their own. No change is expected."),
					Do: func(p *Player) (cardsim.Message, error) {
						p.Stats.BasePopulation -= 20
						return cardsim.MsgStr("A festival of bureaucracy lured away a few kobolds to other nations."), nil
					},
					Undo: func(p *Player) error {
						p.Stats.BasePopulation += 20
						return nil
					},
					CanDo: YesWeCan,
				},
				Variants: []Policy{
					nil,
					&DisabledPolicy{cardsim.MsgStr("Neglecting the event would be less popular and more inefficient than our current policy.")},
					&DisabledPolicy{cardsim.MsgStr("Neglecting the event would be less popular and more inefficient than our current policy.")},
					&FuncPolicy{
						OptionTextFunc: func(p *Player) (cardsim.Message, error) {
							if p.Stats.BureaucracyExpense >= -0.03 {
								return cardsim.MsgStr("Permitting the festival will yield some immigration, but we'll be expected to rebuild our bureaucracy."), nil
							}
							return cardsim.MsgStr("Permitting the festival will yield some immigration."), nil
						},
						EnactFunc: func(p *Player) (cardsim.Message, error) {
							if p.Stats.BureaucracyExpense >= -0.03 {
								return cardsim.MsgStr("A festival of bureaucracy just saw the nation's bureaucracy rebuilt."), ErrKeepMessage
							}
							return cardsim.MsgStr("A festival of bureaucracy brought in immigrants wondering at the nation's lack thereof."), ErrKeepMessage
						},
					},
					nil, // remember, if using a BasicPolicy, this uses EnactedDesc
				},
			},
		}, // End of "International Festival of Bureaucracy" policies
	}, // End of "International Festival of Bureaucracy" card
	&SwitchingCard{ // Card 3
		Name:  cardsim.MsgStr("Two Kobolds in a Trenchcoat"),
		Desc:  cardsim.MsgStr("An inevitable concern of kobold nations is the matter of trade.  Trade with other kobolds is straightforward, but overland caravans are unsafe.  Trade with surfacers is vital... but achieving it is difficult owing to anti-kobold sentiment."),
		After: ShuffleIntoBottomHalf,
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Speaking for the conservative opinion, one prospective merchant says, "We need illusions.  Our priority needs to be trading to acquire or studying to create artifacts of disguising.  The high start-up costs are balanced by free movement in a surface society that frequently detests us."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your merchants rely on magical disguises to avoid being caught as kobolds while doing business on the surface."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.ManufacturingIncome += 0.03
					p.Stats.LogisticsExpense += 0.04
					p.Stats.MiningIncome += 0.01
					p.Stats.AlchemyIncome += 0.01
					p.Stats.GadgetryIncome += 0.01
					p.Stats.ScavengingIncome += 0.01
					p.Stats.FoodSupply += 0.01
					p.Stats.Secrecy += 0.05
					p.Stats.PointOfDimReturns += 0.05
					return cardsim.MsgStr("Nobody ever meets a kobold merchant."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.ManufacturingIncome -= 0.03
					p.Stats.LogisticsExpense -= 0.04
					p.Stats.MiningIncome -= 0.01
					p.Stats.AlchemyIncome -= 0.01
					p.Stats.GadgetryIncome -= 0.01
					p.Stats.ScavengingIncome -= 0.01
					p.Stats.FoodSupply -= 0.01
					p.Stats.Secrecy -= 0.05
					p.Stats.PointOfDimReturns -= 0.05
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Brave but potentially headstrong, another prospective merchant has attended the meeting with the following plea.  "We must take head-on the risk of improving the status of kobolds!  Only by bravely attending the surface markets can we hope to achieve a true and lasting peace!"`),
				EnactedDesc:   cardsim.MsgStr(`[current policy] Your merchants represent the nation's peaceful ideals by visiting the surface openly despite occasional casaulties.`),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 15
					p.Stats.ManufacturingIncome += 0.03
					p.Stats.LogisticsExpense += 0.01
					p.Stats.EconPlanExpense += 0.04
					p.Stats.MiningIncome += 0.01
					p.Stats.AlchemyIncome += 0.02
					p.Stats.GadgetryIncome += 0.02
					p.Stats.HospitalityIncome += 0.01
					p.Stats.PublishingIncome += 0.01
					p.Stats.ForeignRelExpense += 0.01
					p.Stats.WarExpense -= 0.01
					p.Stats.ScavengingIncome -= 0.01
					p.Stats.Secrecy -= 0.10
					p.Stats.ForeignRelations += 0.05
					p.Stats.PointOfDimReturns += 0.1
					return cardsim.MsgStr("The economic planning office formally plans for attrition among its merchants."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 15
					p.Stats.ManufacturingIncome -= 0.03
					p.Stats.LogisticsExpense -= 0.01
					p.Stats.EconPlanExpense -= 0.04
					p.Stats.MiningIncome -= 0.01
					p.Stats.AlchemyIncome -= 0.02
					p.Stats.GadgetryIncome -= 0.02
					p.Stats.HospitalityIncome -= 0.01
					p.Stats.PublishingIncome -= 0.01
					p.Stats.ForeignRelExpense -= 0.01
					p.Stats.WarExpense += 0.01
					p.Stats.ScavengingIncome += 0.01
					p.Stats.Secrecy += 0.10
					p.Stats.ForeignRelations -= 0.05
					p.Stats.PointOfDimReturns += 0.1
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A shady scavenger has offered to change careers and become a merchant.  Their proposal is, "Surfacers aren't all monolithic.  If we talk to the ones who are more greedy than prejudiced, we don't need magic or idealism.  We can get in contact with lots of people who'll buy lots of things from us, even things that might not have a market if we worked with 'good' merchants."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your merchants work with the underworld of the surface, forging connections with the people who are ironically less likely to murder them."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 5
					p.Stats.LogisticsExpense += 0.01
					p.Stats.ScavengingIncome += 0.03
					p.Stats.AlchemyIncome += 0.02
					p.Stats.GadgetryIncome += 0.03
					p.Stats.MiningIncome -= 0.01
					p.Stats.ManufacturingIncome -= 0.01
					p.Stats.HiddenRelPenalty += 0.02 // High is bad on this hidden stat.
					p.Stats.PointOfDimReturns += 0.05
					return cardsim.MsgStr("Kobold merchants are instantly suspected of black market connections."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 5
					p.Stats.LogisticsExpense -= 0.01
					p.Stats.ScavengingIncome -= 0.03
					p.Stats.AlchemyIncome -= 0.02
					p.Stats.GadgetryIncome -= 0.03
					p.Stats.MiningIncome += 0.01
					p.Stats.ManufacturingIncome += 0.01
					p.Stats.HiddenRelPenalty -= 0.02
					p.Stats.PointOfDimReturns -= 0.05
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A mining forebold registered for the right to attend this meeting, and says, "Surfacers can't be trusted in business and we shouldn't be supporting them with trade.  We need underground roads to ease trade with other kobold nations.  It's a massive investment in smoothing current tunnels and digging new ones, but think of the reward: a world of underground civilizations supporting each other."`),
				EnactedDesc:   cardsim.MsgStr(`[currentpolicy] Your merchants have been directed to focus on trade with other kobolds, and your miners are perpetually tunnelling towards other kobold nations in pursuit of the dream of a world of underground roads.`),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.MiningIncome += 0.05
					p.Stats.ConstructionIncome += 0.01
					p.Stats.LogisticsExpense += 0.06
					p.Stats.FoodSupply -= 0.01
					p.Stats.Secrecy += 0.15
					p.Stats.PointOfDimReturns += 0.1
					return cardsim.MsgStr("The vast network of caverns under the surface of the world is not a natural phenomenon."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.MiningIncome -= 0.05
					p.Stats.ConstructionIncome -= 0.01
					p.Stats.LogisticsExpense -= 0.06
					p.Stats.FoodSupply += 0.01
					p.Stats.Secrecy -= 0.15
					p.Stats.PointOfDimReturns -= 0.1
					return nil
				},
				CanDo: YesWeCan,
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("Refusing an opportunity to expand trade is merely an opportunity cost.  It won't make conditions any better or worse."),
					EnactedDesc:   cardsim.MsgStr("Refusing an opportunity to expand trade is merely an opportunity cost.  It won't make conditions any better or worse."),
					Do: func(p *Player) (cardsim.Message, error) {
						return cardsim.MsgStr("Kobold merchants are known for their talent at standing on each others' shoulders."), nil
					},
					Undo: func(p *Player) error {
						return nil
					},
					CanDo: YesWeCan,
				},
				Variants: []Policy{
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Rejecting the illusion-based merchantry will cost us a great deal in trade and surface food, but we can certainly do it."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Rejecting our idealistic approach to surface trade will save a few lives directly, but consider how much we need the food we buy from the surfacers."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("The surfacers' governments would certainly prefer us to abandon our underworld connections in their cities.  Do we care about that?  We'd be giving up some excellent profits."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Some of the miners would be heartbroken by abandoning the dream of underground roads, but the tax savings would be significant.  Reducing our trade focus on other kobolds does however carry some risk."),
					},
				},
			},
		}, // End of "Two Kobolds in a Trenchcoat" policies
	}, // End of "Two Kobolds in a Trenchcoat" card
	&SwitchingCard{ // Card 4
		Name:  cardsim.MsgStr("A Recipe for Stewed Kobold"),
		Desc:  cardsim.MsgStr("Nobody would ever say a kobold smells bad, but the constant burrowing, manufacturing, and metallurgy that is common to kobold life is still a recipe for tense muscles and filthy scales.  A slate of proposals have come through to address this issue."),
		After: ShuffleIntoBottomHalf,
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A calm and cheerful scribe says, "We should have public baths!  We have more problems with flooding than drought in the deep underground, so we've got plenty of water.  We just need to channel it.  We could create a hub of social life while improving the health of everybody!"`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Public bathing is a staple social policy in the mines and warrens of your nation."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.ConstructionIncome += 0.01
					p.Stats.QoLExpense += 0.01
					p.Stats.HealthcareExpense += 0.01
					p.Stats.ParksExpense += 0.04
					return cardsim.MsgStr("The company of others makes cold water tolerable."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.ConstructionIncome -= 0.01
					p.Stats.QoLExpense -= 0.01
					p.Stats.HealthcareExpense -= 0.01
					p.Stats.ParksExpense -= 0.04
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your Minister of Finance is scandalized.  "A free service?  And such an expensive one to construct?  No, no, no.  The baths must pull their weight in the budget if they're ot be made at all.  We must set an entry fee."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Elite bathhouses serve paying customers in your largest sites."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.ConstructionIncome += 0.01
					p.Stats.HospitalityIncome += 0.01
					p.Stats.ParksExpense += 0.02
					return cardsim.MsgStr("A dirty body is the mark of the lower classes."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.ConstructionIncome -= 0.01
					p.Stats.HospitalityIncome -= 0.01
					p.Stats.ParksExpense -= 0.02
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your head diplomat chimes in with, "If you want to make the service self-sustaining, build a bath in a shallow cave and invite the surfacers in for a fee.  Of course, we'll be giving up some of our secrecy, but it'll bring in foreign exchange to power our industries."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Conservative kobolds are scandalized: not only has the nation bathhouses, it has near-surface bathhouses for surfacers to visit."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.ConstructionIncome += 0.02
					p.Stats.HospitalityIncome += 0.03
					p.Stats.ManufacturingIncome += 0.01
					p.Stats.ParksExpense += 0.03
					p.Stats.FoodSupply += 0.01
					p.Stats.Secrecy -= 0.10
					return cardsim.MsgStr("Surfacers stink less in the nation's vicinity."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.ConstructionIncome -= 0.02
					p.Stats.HospitalityIncome -= 0.03
					p.Stats.ManufacturingIncome -= 0.01
					p.Stats.ParksExpense -= 0.03
					p.Stats.FoodSupply -= 0.01
					p.Stats.Secrecy += 0.10
					return nil
				},
				CanDo: YesWeCan,
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("Bathhouses are a big project that would displace other things in the city center.  You could use the space for housing and services instead."),
					EnactedDesc:   cardsim.MsgStr("When is the right time to embark on building public baths?  You can hew close to skeptics instead."),
					Do: func(p *Player) (cardsim.Message, error) {
						p.Stats.BasePopulation += 5
						return cardsim.MsgStr("Opponents of public bathing won a recent political contest."), nil
					},
					Undo: func(p *Player) error {
						p.Stats.BasePopulation -= 5
						return nil
					},
					CanDo: YesWeCan,
				},
			},
		}, // End of "A Recipe for Stewed Kobold" policies
	}, // End of "A Recipe for Stewed Kobold" card
	&SwitchingCard{ // Card 5
		Name:  cardsim.MsgStr("Raid on Surface Proposed"),
		Desc:  cardsim.MsgStr("Resources underground are perpetually tight.  Even metallurgy is hard, because fuel is scarce.  You can rely on neither field nor forest.  The scavengers brave the surface regularly to beg, borrow, or steal the resources necessary to your society, but your war planners want to secure resources more aggressively."),
		After: ShuffleIntoBottomHalf,
		IsValid: func(c Card, p *Player) bool {
			return p.Stats.WarExpense > 0
		},
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A scavenger clad in sturdy clothes says, "We need to secure a section of forests on the surface.  The surfacers strip too many of the forests bare; if we dispatch patrols ready to fight we can keep them from assarting the forest for their endlessly growing farm communities."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] The war department maintains your meagre conquest of a stretch of forest on the surface."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 20
					p.Stats.ForestryIncome += 0.03
					p.Stats.WarExpense += 0.01
					p.Stats.FoodSupply += 0.03
					p.Stats.Secrecy -= 0.10
					p.Stats.ForeignRelations -= 0.02
					return cardsim.MsgStr("A bunch of lumberjacks just disappeared in a forest."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 20
					p.Stats.ForestryIncome -= 0.03
					p.Stats.WarExpense -= 0.01
					p.Stats.FoodSupply -= 0.05
					p.Stats.Secrecy += 0.10
					p.Stats.ForeignRelations += 0.02
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A very scrawny warrior waves a spear over their head and says, "That's not going far enough!  We need to seize farmland of our own!  Come on, we shouldn't have to eat cave lichen while the surfacers eat roast pig!"`),
				EnactedDesc:   cardsim.MsgStr("[current policy] The 'ghost farms' in this region are worked by night as kobolds sparing their eyes avoid doing farmwork in daylight."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 40
					p.Stats.AgricultureIncome += 0.01
					p.Stats.WarExpense += 0.02
					p.Stats.FoodSupply += 0.06
					p.Stats.Secrecy -= 0.20
					p.Stats.ForeignRelations -= 0.04
					return cardsim.MsgStr("Some farmers fight like five kobolds, but few fight like forty."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 40
					p.Stats.AgricultureIncome -= 0.01
					p.Stats.WarExpense -= 0.02
					p.Stats.FoodSupply -= 0.06
					p.Stats.Secrecy += 0.20
					p.Stats.ForeignRelations += 0.04
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A warrior kobold of quiet intensity says, "We shouldn't try to hold terrain on the surface.  We'll give ourselves away to surfacers who otherwise can't pinpoint us.  Remember that we can't stand against their armies.  What we can do without violating secrecy is focus on banditry.  We've mapped activity in our area; it's time to go in for a harvest of our own."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Your warriors target caravans for intimidation and larceny."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 10
					p.Stats.ScavengingIncome += 0.02
					p.Stats.WarExpense += 0.01
					p.Stats.FoodSupply += 0.01
					p.Stats.Secrecy -= 0.05
					p.Stats.HiddenRelPenalty += 0.05
					// TODO: Subtract a compassion point.
					return cardsim.MsgStr("Some merchants are even cordial as they pay off kobold bandits."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 10
					p.Stats.ScavengingIncome -= 0.02
					p.Stats.WarExpense -= 0.01
					p.Stats.FoodSupply -= 0.01
					p.Stats.Secrecy += 0.05
					p.Stats.HiddenRelPenalty -= 0.05
					return nil
				},
				CanDo: YesWeCan,
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("Rejecting militarism will set back the war department, but hunger will persist."),
					EnactedDesc:   cardsim.MsgStr("[current policy] The war department has been rebuked for its aggression."),
					Do: func(p *Player) (cardsim.Message, error) {
						p.Stats.Secrecy += 0.01
						p.Stats.WarExpense -= 0.01
						p.Stats.HiddenRelPenalty -= 0.01
						return cardsim.MsgStr("Some of the warriors have had their fervor redirected into economic pursuits."), nil
					},
					Undo: func(p *Player) error {
						p.Stats.Secrecy -= 0.01
						p.Stats.WarExpense += 0.01
						p.Stats.HiddenRelPenalty += 0.01
						return nil
					},
					CanDo: YesWeCan,
				},
			},
		}, // End of "Raid on Surface Proposed" policies
	}, // End of "Raid on Surface Proposed" card
	&SwitchingCard{ // Card 6
		Name:  cardsim.MsgStr("Book Publishing Threat to Secrecy?"),
		Desc:  cardsim.MsgStr("Concerns have been raised about one of the categories of income that your nation relies upon: books.  To a lesser extent, art in general.  Some artifacts, especially books, have the potential to give away details of your peoples' lives that help hostile powers locate your nation."),
		After: ShuffleIntoBottomHalf,
		IsValid: func(c Card, p *Player) bool {
			return p.Stats.PublishingIncome > 0 && p.Stats.ForeignRelations < 0
		},
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`The warrior who brought this to your attention stands at attention as they say, "The risk of discovery is an existential risk!  Everything that makes us discoverable increases the intensity of conflict around our warrens, draining lives from our people!  It is imperative that all respect our code of silence!"`),
				EnactedDesc:   cardsim.MsgStr("[current policy] What published material emanates from your nation is strictly censored of details that could indicate the locations of your homes."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.Secrecy += 0.03
					p.Stats.PublishingIncome -= 0.05
					p.Stats.EducationExpense -= 0.01
					return cardsim.MsgStr("No history records the sobbing of kobold scribes."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.Secrecy -= 0.03
					p.Stats.PublishingIncome += 0.05
					p.Stats.EducationExpense += 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`"No!  Never!  Er, maybe?" asks a dithering scribe, clutching a book worriedly to their chest.  "Can we set up an office of censorship to read everything we publish and send it back for edits if it would reveal anything about our location?  We don't have to stop publishing; our traders buy food with foreign currencies you know!"`),
				EnactedDesc:   cardsim.MsgStr("[current policy] The bureaucracy includes an office of censorship that reads all writings produced for sale to ensure that nothing is revealed about the location of any living kobolds."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.Secrecy += 0.01
					p.Stats.PublishingIncome -= 0.01
					p.Stats.BureaucracyExpense += 0.01
					return cardsim.MsgStr("The censors are the most well-read kobolds in the nation."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.Secrecy -= 0.01
					p.Stats.PublishingIncome += 0.01
					p.Stats.BureaucracyExpense -= 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your Minister of Education wrings his claws sadly.  "I know it'll be a monumental expense in paper and ink, but we need to encourage the writing of diaries.  Our secrecy is already flawed; there are already attacks against us.  Maybe our attackers will find diaries and understand that we aren't just thieves hiding under the earth."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Even though recovered diaries point back to your outposts, your kobolds are encouraged to carry around diaries and record thoughts from their own lives into them."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.Secrecy -= 0.01
					p.Stats.PublishingIncome += 0.02
					p.Stats.EducationExpense += 0.03
					p.Stats.PropagandaExpense += 0.01
					p.Stats.HiddenRelPenalty -= 0.01
					p.Stats.PointOfDimReturns += 0.01
					// TODO: Add a compassion point
					return cardsim.MsgStr("Heartless surfacers burn recovered kobold diaries."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.Secrecy += 0.01
					p.Stats.PublishingIncome -= 0.02
					p.Stats.EducationExpense -= 0.03
					p.Stats.PropagandaExpense -= 0.01
					p.Stats.HiddenRelPenalty += 0.01
					p.Stats.PointOfDimReturns -= 0.01
					return nil
				},
				CanDo: func(b *BasicPolicy, p *Player) bool {
					return p.Stats.Secrecy < 1
				},
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("Rejecting this will inspire celebrations among the authors of your nation."),
					EnactedDesc:   cardsim.MsgStr("The dangers posed by publishing information from your nation can be denied without changes in policy."),
					Do: func(p *Player) (cardsim.Message, error) {
						p.Stats.PublishingIncome += 0.01
						return cardsim.MsgStr("Kobold-hating adventurers know to be well-read."), nil
					},
					Undo: func(p *Player) error {
						p.Stats.PublishingIncome -= 0.01
						return nil
					},
					CanDo: YesWeCan,
				},
				Variants: []Policy{
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Your publishing industry is struggling under harsh restrictions and would soar if relieved."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("The censorship office spends tax money to crimp tax income.  It has opponents, though they stay quiet."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Rescinding the diary initiative would ding literacy rates, but it should save on taxes."),
					},
				},
			},
		}, // End of "Book Publishing Threat to Security?" policies"
	}, // End of "Book Publishing Threat to Security?" card
	&SwitchingCard{ // Card 7
		Name:  cardsim.MsgStr("The Eyes that Break the Ground"),
		Desc:  cardsim.MsgStr("Some of our miners have recently been afflicted with shared hallucinations.  They report feelings of being watched in dark areas - which, thanks to kobold darkvision, is nearly everywhere - as well as seeing black blotches covered in eyes on various surfaces.  They're refusing to continue work."),
		After: ShuffleIntoBottomHalf,
		Policies: []Policy{
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`A friend of one of the afflicted managed to get onto your meeting schedule to say, "We work our miners hard.  We need to give them more breaks.  We need to fund more investments into healthcare, too.  We can set our alchemists to work brewing potions that should help."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Afflicted miners are being dosed with soothing alchemical teas."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.MiningIncome -= 0.01
					p.Stats.AlchemyIncome += 0.02
					p.Stats.HealthcareExpense += 0.01 // TODO:  Implement population boost from Healthcare investment.  Maybe 1000x HealthcareInvestment, so that each 0.01 Healthcare Investment is a bonus of 10 kobolds to the population?
					p.Stats.PointOfDimReturns += 0.02
					// Note to self, add a compassion point.
					return cardsim.MsgStr("Calming alchemical potions are displacing raw material sales in local markets."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.MiningIncome += 0.01
					p.Stats.AlchemyIncome -= 0.02
					p.Stats.HealthcareExpense -= 0.01
					p.Stats.PointOfDimReturns -= 0.02
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`Your chief economic planner regards the report with surprise and disgust.  "This is just an excuse for slacking!  Send them back into the mines and hire guards to make sure they double production!  And if they see any more eyes in the dark, tell 'em to shatter 'em with detonation charges!"`),
				EnactedDesc:   cardsim.MsgStr("[current policy] The mines are taut as crazy miners with explosives work under the oversight of armed guards."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 10
					p.Stats.MiningIncome += 0.03
					p.Stats.GadgetryIncome += 0.01
					p.Stats.WarExpense += 0.01
					p.Stats.PointOfDimReturns -= 0.01
					return cardsim.MsgStr("Miners are known for their enthusiasm for explosives."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.MiningIncome -= 0.03
					p.Stats.GadgetryIncome -= 0.01
					p.Stats.WarExpense -= 0.01
					p.Stats.PointOfDimReturns += 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`The head of your Ministry of Soothsaying, Shamanism, and Otherworldly Affairs has reacted with unsettling enthusiasm.  "Shared hallucinations?  Of eyes in the dark?  This is an opportunity!  We just need to train the afflicted into new spellcasters.  We could learn the kind of secrets that shatter minds!"`),
				EnactedDesc:   cardsim.MsgStr("[current policy] Mine policy is to watch for this symptom and trial the afflicted in planar summoning."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 20
					p.Stats.MiningIncome -= 0.01
					p.Stats.PlanarIncome += 0.03
					p.Stats.ResearchSubsExpense += 0.01
					p.Stats.PointOfDimReturns += 0.02
					// TODO: Add more consequences to questionable planar contacts.
					return cardsim.MsgStr("The nation is infamous for producing crazed magi."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 20
					p.Stats.MiningIncome += 0.01
					p.Stats.PlanarIncome += 0.03
					p.Stats.ResearchSubsExpense -= 0.01
					p.Stats.PointOfDimReturns -= 0.02
					return nil
				},
				CanDo: YesWeCan,
			},
			&BasicPolicy{
				UnenactedDesc: cardsim.MsgStr(`The manager of one of the affected signed the right papers to get into this meeting.  Regarding you coldly, they say, "They're slacking, and it's not right to permit anything to change.  We don't soothing alchemicals, we don't need new guards, adding explosives to the mix is just chaotic, and recruiting crazies as spellcasters is insane itself.  Trust in the miners to enforce discipline amongst each other.  We don't permit anyone to hold up production."`),
				EnactedDesc:   cardsim.MsgStr("[current policy] The miners are dealing with laggard elements on their own."),
				Do: func(p *Player) (cardsim.Message, error) {
					p.Stats.BasePopulation -= 20
					p.Stats.MiningIncome += 0.01
					p.Stats.PointOfDimReturns += 0.01
					// TODO: Take away a point of compassion for this.
					return cardsim.MsgStr("Sufficient misery unmakes people."), nil
				},
				Undo: func(p *Player) error {
					p.Stats.BasePopulation += 10
					p.Stats.MiningIncome -= 0.01
					p.Stats.PointOfDimReturns -= 0.01
					return nil
				},
				CanDo: YesWeCan,
			},
			&VerbosePolicy{
				Default: &BasicPolicy{
					UnenactedDesc: cardsim.MsgStr("Maybe we can just work around the afflicted.  Let them rest and find other employment."),
					EnactedDesc:   cardsim.MsgStr("[current policy] We're encouraging the afflicted to rest and find other employment."),
					Do: func(p *Player) (cardsim.Message, error) {
						p.Stats.MiningIncome -= 0.01
						p.Stats.PointOfDimReturns += 0.01
						// TODO:  Add a compassion point.
						return cardsim.MsgStr("Mining is just too strenuous for some kobolds."), nil
					},
					Undo: func(p *Player) error {
						p.Stats.MiningIncome += 0.01
						p.Stats.PointOfDimReturns -= 0.01
						return nil
					},
					CanDo: YesWeCan,
				},
				Variants: []Policy{
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Rescinding the healthcare subsidy might ding our population, but it shouldn't ding our compassion.  We're being very gentle with these people."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Withdrawing explosives access from the suffering may be a life-saving action..."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("Maybe we should close the door on this particular otherworldly contact source."),
					},
					&BasicPolicy{
						UnenactedDesc: cardsim.MsgStr("The miners' culture is harsh!  There's an argument here for protecting the suffering."),
					},
					nil,
				},
			},
		}, // end of "The Eyes that Break the Ground" policies
	}, // end of "The Eyes that Break the Ground" card
} // end of card list

func initDeck(d *cardsim.Deck[*KoboldMine]) {
	for _, c := range cards {
		d.Insert(cardsim.BottomOfDeck, c)
	}
	d.Shuffle()
}
